#!/usr/bin/env python
#
# [SNIPPET_NAME: Drawing With Cairo]
# [SNIPPET_CATEGORIES: PyGTK, Cairo]
# [SNIPPET_DESCRIPTION: Create a drawing with cairo]
# [SNIPPET_AUTHOR: Jurjen Stellingwerff <jurjen@stwerff.xs4all.nl>]
# [SNIPPET_LICENSE: GPL]

# example cairo_demo.py
""" Example code for pyCairo """

import pygtk
pygtk.require('2.0')
import gtk
import math
import cairo

full = False
window = None
context = None


def drawingAreaExample():
    """ Add basic elements to the window """
    global window
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.set_title("Drawing With Cairo")
    window.connect("destroy", lambda w: gtk.main_quit())
    area = gtk.DrawingArea()
    area.set_size_request(380, 300)
    area.connect("button-press-event", cb_mouse_click)
    area.set_events(gtk.gdk.BUTTON_PRESS_MASK)
    window.add(area)
    area.connect("expose-event", expose)
    area.show()
    window.show()


def cb_mouse_click(area, event):
    """ Handle mouse events, toggle to fullscreen """
    global full
    if event.button == 1:
        if full:
            window.unfullscreen()
            full = False
        else:
            window.fullscreen()
            full = True
        print "Mouse click ", event.x, event.y


def expose(area, event):
    """ General routine to redraw the window """
    global context
    context = area.window.cairo_create()
    context.rectangle(event.area.x, event.area.y, event.area.width,
            event.area.height)
    context.clip()
    context.scale(event.area.width / 380.0,
                           event.area.height / 300.0)
    draw_point(10, 10)
    draw_points(110, 10)
    draw_line(210, 10)
    draw_lines(310, 10)
    draw_segments(10, 100)
    draw_rectangles(110, 100)
    draw_arcs(210, 100)
    draw_pixmap(310, 100)
    draw_polygon(10, 200)
    draw_rgb_image(110, 200)
    context.move_to(210, 290)
    context.show_text("Try to resize the window")
    return False


def point(x, y):
    """ draw a point, or a small circle """
    context.arc(x, y, 1, 0.0, 2.0 * math.pi)
    context.fill()


def draw_point(x, y):
    """ draw a point example """
    point(x + 30, y + 30)
    context.move_to(x + 5, y + 50)
    context.show_text("Point")


def draw_points(x, y):
    """ draw a few points example """
    points = [(x + 10, y + 10), (x + 10, y), (x + 40, y + 30),
                 (x + 30, y + 10), (x + 50, y + 10)]
    for px, py in points:
        point(px, py)
    context.move_to(x + 5, y + 50)
    context.show_text("Points")


def draw_line(x, y):
    """ draw a line example """
    context.move_to(x + 10, y + 10)
    context.line_to(x + 20, y + 30)
    context.stroke()
    context.move_to(x + 5, y + 50)
    context.show_text("Line")


def draw_lines(x, y):
    """ draw a couple of lines examples """
    points = [(x + 10, y), (x + 40, y + 30),
                  (x + 30, y + 10), (x + 50, y + 10)]
    context.move_to(x + 10, y + 10)
    for px, py in points:
        context.line_to(px, py)
    context.set_line_width(4.5)
    context.stroke()
    context.save()
    context.set_source_rgb(1, 1, 0.7)
    context.translate(1.5, 1)
    context.move_to(x + 10, y + 10)
    for px, py in points:
        context.line_to(px, py)
    context.set_line_width(1.5)
    context.stroke()
    context.restore()
    context.set_line_width(2)
    context.move_to(x + 5, y + 50)
    context.show_text("Lines")


def draw_segments(x, y):
    """ draw some straight lines """
    segments = ((x + 20, y + 10, x + 20, y + 70),
                (x + 60, y + 10, x + 60, y + 70),
                (x + 10, y + 30, x + 70, y + 30),
                (x + 10, y + 50, x + 70, y + 50))
    for fx, fy, tx, ty in segments:
        context.move_to(fx, fy)
        context.line_to(tx, ty)
    context.stroke()
    context.move_to(x + 5, y + 90)
    context.show_text("Segments")


def draw_rectangles(x, y):
    """ draw some boxes """
    context.save()
    context.set_source_rgb(.6, 0, 0)
    context.rectangle(x, y, 80, 70)
    context.stroke()
    context.restore()
    rectangles = ((x + 10, y + 10, 20, 20), (x + 50, y + 10, 20, 20),
                  (x + 20, y + 50, 40, 10))
    for px, py, h, w in rectangles:
        context.rectangle(px, py, h, w)
        context.fill()
    context.move_to(x + 5, y + 90)
    context.show_text("Rectangles")


def draw_arcs(x, y):
    """ draw a silly face """
    context.set_line_width(1)
    arcs = [(False, x + 45, y + 35, 35),
            (True, x + 35, y + 25, 5),
            (True, x + 55, y + 25, 5)]
    for fill, px, py, r in arcs:
        context.move_to(px + r, py)
        context.arc(px, py, r, 0, 2 * math.pi)
        if fill:
            context.fill()
        else:
            context.set_source_rgb(1, 1, 0.5)
            context.fill_preserve()
            context.set_source_rgb(0, 0, 0)
            context.stroke()
    context.move_to(x + 45, y + 35)
    s = 1.66
    context.save()
    context.scale(1, s)
    context.arc(x + 45, (y + 35) / s, 15, 30 * math.pi / 180,
                150 * math.pi / 180)
    context.restore()
    context.save()
    context.move_to(x + 5, y + 90)
    context.select_font_face("serif", cairo.FONT_SLANT_NORMAL,
                                      cairo.FONT_WEIGHT_BOLD)
    context.show_text("Arcs")
    context.restore()


def draw_pixmap(x, y):
    """ show an image """
    try:
        img = cairo.ImageSurface.create_from_png("gtk.png")
        context.save()
        context.set_source_surface(img, x + 5, y + 15)
        context.paint()
        context.restore()
        context.move_to(x + 5, y + 90)
        context.show_text("Pixmap")
    except cairo.Error, e:
        print e


def draw_polygon(x, y):
    """ a filled irregular shape """
    points = [(x + 10, y + 20), (x + 40, y + 70),
              (x + 30, y + 30), (x + 50, y + 40)]
    context.move_to(x + 10, y + 60)
    for px, py in points:
        context.line_to(px, py)
    context.fill()
    context.move_to(x + 5, y + 90)
    context.show_text("Polygon")


def draw_rgb_image(x, y):
    """ show a box filled with gradients """
    s = 80
    context.save()
    linpat = cairo.LinearGradient(x, y, x + s, y + s)
    linpat.add_color_stop_rgb(0.0, 1, 1, 1)
    linpat.add_color_stop_rgb(1.0, 0, 1, 0)
    context.set_source(linpat)
    context.move_to(x, y)
    context.line_to(x, y + s)
    context.line_to(x + s, y + s)
    context.line_to(x + s, y)
    context.fill()
    linpat = cairo.LinearGradient(x, y + s, x + s, y)
    linpat.add_color_stop_rgba(0.0, 0, 0, 1, 1)
    linpat.add_color_stop_rgba(0.5, 1, 0, 1, 0)
    linpat.add_color_stop_rgba(1.0, 1, 0, 0, 1)
    context.set_source(linpat)
    context.move_to(x, y)
    context.line_to(x, y + s)
    context.line_to(x + s, y + s)
    context.line_to(x + s, y)
    context.fill()
    context.restore()
    context.save()
    context.move_to(x + 5, y + 92)
    context.set_font_size(11.5)
    context.show_text("RGB Image")
    context.restore()


def main():
    """ start the program main loop """
    gtk.main()
    return 0

if __name__ == "__main__":
    drawingAreaExample()
    main()
