""" Interface elements """
import pygtk
pygtk.require('2.0')
import gtk
import gconf
import time
import gobject

cb_click = {}  # known handlers for mouse clicks
cb_render = {}  # known handlers for redrawing screens
cb_timer = {}  # known handlers for timeouts
linePos = 0  # current position for on screen texts
lastX = 0  # last x position for on screen texts
timeout = 3

context = None
area = None
window = None
page = None


def register(page_name, click, render, timer=None):
    """ Register a known page """
    cb_click[page_name] = click
    cb_render[page_name] = render
    if timer != None:
        cb_timer[page_name] = timer


def set_timeout(ticks):
    """ Set the timout """
    global timeout
    timeout = ticks


def cb_timeout_routine():
    """ Timeout routine """
    global timeout, page
    if timeout > 0:
        timeout -= 1
    else:
        page = None
        redraw_all()
        screensaver(True)
    return True


def minimize():
    """ Minimize the screen """
    window.unfullscreen()
    window.iconify()


def line_set(pos, last):
    """ Set the next text line position """
    global linePos, lastX
    linePos = pos
    lastX = last


def line(x, t):
    """ Write a line and update position """
    global linePos, lastX
    context.set_font_size(17)
    if x < lastX:
        linePos += 20
    lastX = x
    context.move_to(x, linePos)
    context.show_text(str(t))


def redraw_area(fx, fy, tx, ty):
    """ Redraw a part of the screen """
    area.queue_draw_area(fx, fy, tx, ty)


def redraw_all():
    """ Redraw the whole screen """
    area.queue_draw()


def draw_text(t, pt, maxw):
    """ Write text with size and maximal width """
    context.set_font_size(pt)
    width = context.text_extents(t)[4]
    if width > maxw:
        context.set_font_size(pt / width * maxw)
    else:
        context.rel_move_to((maxw - width) / 2, 0)
    context.show_text(t)


def draw_nr(nr, pos):
    """ Write a number """
    cl = len(str(int(nr)))
    context.rel_move_to((pos - cl) * 11, 0)
    if isinstance(nr, float):
        sn = "%.2f" % nr
    else:
        sn = str(nr)
    context.show_text(sn)


def screensaver(state):
    """ Disable or enable the screensaver """
    client = gconf.client_get_default()
    client.set_bool("/apps/gnome-screensaver/idle_activation_enabled", state)


def cb_mouse(click_area, event):
    """ General mouse click handler """
    global page, timeout
    timeout = 3
    screensaver(False)
    window.present()
    if event.button == 1:
        new_page = cb_click[page](event.x, event.y)
        if new_page != page:
            page = new_page
            redraw_all()


def cb_expose(expose_area, event):
    """ Redrawing event handler """
    global context
    context = expose_area.window.cairo_create()
    context.set_source_rgb(0, 0, 0)
    context.paint()
    cb_render[page](context)
    context.flush()

lasttime = time.time()


def cb_timeout():
    """ Perform timed functions """
    global lasttime
    try:
        if time.time() < lasttime:
            return True
        cb_timer[page](area)
        lasttime += 0.02
    except KeyError:
        return False
    return True


def drawingArea():
    """ Window handlers """
    global area, window
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.set_title("Title")
    window.connect("destroy", gtk.main_quit)
    area = gtk.DrawingArea()
    area.connect('expose-event', cb_expose)
    area.set_size_request(800, 600)
    area.connect("button-press-event", cb_mouse)
    area.set_events(gtk.gdk.BUTTON_PRESS_MASK)
    window.add(area)
    area.show()
    window.show()
    #gobject.timeout_add(20, cb_timeout)
    gobject.idle_add(cb_timeout)

def main():
    """ start the program main loop """
    gtk.main()
    return 0
