""" Sorting objects efficiently """
from operator import itemgetter, attrgetter, methodcaller


class Person(object):
    """ Example person """
    __slots__ = ('name', '_age')

    def __init__(self, name, age):
        self.name = name
        self._age = age

    def __repr__(self):
        return self.name

    def __getitem__(self, nr):
        return self.name if nr == 0 else self.age

    def mixed(self):
        """ Mixed data with first token from name and age """
        return self.name[:1] + "%02i" % self.age

    def set_age(self, age):
        """ Controlled setting the age """
        if age < 8:
            raise ValueError("Too young")
        if age > 100:
            raise ValueError("Too old")
        self._age = age

    def get_age(self):
        """ Getting the age """
        return self._age

    age = property(get_age, set_age)

p = [Person("Mike", 9), Person("Maurice", 15),
     Person("Irlene", 17), Person("Mike", 25), Person("Otto", 17)]

print "Set age too young"
try:
    p[0].age = 5
except ValueError:
    pass

print "Mixed:", p[0].name, p[0].mixed(),
for d in p[1:]:
    print ',', d.name, d.mixed(),
print

print "Sorting:", sorted(p, key=itemgetter(0, 1)),
print sorted(p, key=attrgetter("age", "name")),
print sorted(p, key=methodcaller('mixed'))

import cPickle

f = open('/tmp/data', 'wb')
cPickle.dump(p, f, 2)
f.close()

f = open('/tmp/data', 'rb')
n = cPickle.load(f)
f.close()
print 'Reread:', n
