'Example game'
import interface
import cairo
import math
import copy

dist = 0
max_x = 0
max_y = 0


class Line(object):
    'Definition of a line'
    __slots__ = ('x', 'y', 'dx', 'dy', 'dr', 'direction')

    def __init__(self, x, y, dx, dy):
        self.x = x
        self.y = y
        self.dx = dx
        self.dy = dy
        self.dr = math.sqrt(dx * dx + dy * dy)
        self.direction = 'h'

    def normalize(self, l):
        'Normalize the length of the distance vector to l'
        self.dx /= self.dr / l
        self.dy /= self.dr / l
        self.dr = l

    def move(self, l):
        'Move l times the distance vector along the line'
        return (self.x + self.dx * l, self.y + self.dy * l)

    def intersect(self, x, y, r):
        'Calculate the intesection of the line with a circle'
        if x < 25 or x > max_x - 25 or y < 25 or y > max_y - 25:
            return 9999
        l = self
        a = l.dr * l.dr
        b = 2 * (l.x - x) * l.dx + 2 * (l.y - y) * l.dy
        c = (l.x - x) * (l.x - x) + (l.y - y) * (l.y - y) - r * r
        di = b * b - 4 * a * c
        if di < 0:
            return 9999
        di = math.sqrt(di)
        t1 = (-b + di) / a / 2
        t2 = (-b - di) / a / 2
        return min(t1, t2)

    def block(self, x, y, w, h):
        'Calculate the insersection of the line with a rectangle'
        l = self
        tw = w if l.dx < 0 else 0
        th = h if l.dy < 0 else 0
        t1 = 9999
        t2 = 9999
        if l.dx != 0:
            t1 = (x + tw - l.x) / l.dx
            y1 = l.y + t1 * l.dy
            if y1 < y or y1 > y + h or t1 < 0.5:
                t1 = 9999
        if l.dy != 0:
            t2 = (y + th - l.y) / l.dy
            x2 = l.x + t2 * l.dx
            if x2 < x or x2 > x + w or t2 < 0.5:
                t2 = 9999
        return (min(t1, t2), 'v' if t1 > t2 else 'h')

    def distance(self, x, y):
        'Calculate the distance of a line with a point'
        l = self
        return abs(l.dx * (l.y - y) - (l.x - x) * l.dy) / l.dr

    def boxed(self, lx, ly, mx, my):
        'Calculate the distance of a line to an outside box'
        if self.dx > 0:
            x = (mx - self.x) / self.dx
        elif self.dx < 0:
            x = (lx - self.x) / self.dx
        else:
            x = 9999
        if self.dy > 0:
            y = (my - self.y) / self.dy
        elif self.dy < 0:
            y = (ly - self.y) / self.dy
        else:
            y = 9999
        if x < y:
            line.direction = 'h'
        else:
            line.direction = 'v'
        return min(x, y)

    def bounce(self, d):
        'Calculate new direction of the object after bouncing a wall'
        l = self
        l.x, l.y = l.move(d)
        if l.direction == 'v':
            l.dy = -l.dy
        else:
            l.dx = -l.dx


def distance(x1, y1, x2, y2):
    'Calculate the distance of two points'
    dx = x2 - x1
    dy = y2 - y1
    return math.sqrt(dx * dx + dy * dy)


line = Line(0, 0, 0, 0)
sc = 0


def cb_render(ctx):
    'Show the level elements'
    ctx.scale(sc, sc)
    ctx.move_to(0, 0)
    ctx.set_source_rgb(0, 0, 0)
    for px, py in [(max_x, 0), (max_x, max_y), (10, max_y), (10, max_y - 10),
                   (max_x - 10, max_y - 10), (max_x - 10, 10), (10, 10),
                   (10, max_y), (0, max_y), (0, 0)]:
        ctx.line_to(px, py)
    ctx.fill()
    r = 0
    for l in level:
        c = 0
        for t in l:
            if t != '\n' and t != '.' and t != ' ' and t != '^':
                show(ctx, balls[int(t)], c * 30 + 10,
                     r * 50 + 10 + (7 if t == '0' else 0))
            if t == '^':
                show(ctx, star, c * 30 + 19, r * 50 - 32)
            c += 1
        r += 1
    show_cannon(ctx)


def measure():
    'Show the current level measurements'
    global max_x, max_y, sc
    max_x = 800
    max_y = 900
    g = interface.window.get_size()
    sc = min(float(g[0]) / max_x, float(g[1]) / max_y)


def show_cannon(ctx):
    'Show the cannon'
    global line, dist
    mouse = interface.window.get_pointer()
    ctx.save()
    matrix = cairo.Matrix(1, 0, 0, 1, 0, 0)
    matrix.translate(max_x / 2, max_y - 56)
    dx = mouse[0] - max_x * sc / 2
    dy = mouse[1] - (max_y - 56) * sc
    matrix.rotate(-math.atan(dx / float(dy)))
    ctx.transform(matrix)
    show(ctx, cannon, -48, -84)
    show(ctx, balls[1], -30, -136)
    ctx.restore()
    show(ctx, balls[2], max_x / 2 - 30, max_y - 86)
    remember = copy.deepcopy(line)
    odist = dist
    if dist > 0:
        result = calc_cannon(ctx)
        while result == None:
            cannon_line(ctx, dist)
            line.bounce(dist)
            dist = line.boxed(35, 35, max_x - 35, max_y - 35)
            result = calc_cannon(ctx)
        ctx.move_to(line.x, line.y)
        line_to(ctx, result[0], result[1])
        ctx.stroke()
    line = copy.deepcopy(remember)
    dist = odist


def calc_cannon(ctx):
    'Calculate the next end position of the ball'
    global dist
    horiz = abs(line.dx) > abs(line.dy)
    ctx.set_source_rgb(0.5, 0, 0.5)
    for i in range(0, int(dist + 1)):
        x, y = line.move(i)
        t = []
        col, row = grid(x, y)
        for p in range(-2, 2):
            if horiz:
                c = col - (0 if p % 2 == 0 else -1)
                r = row + p
            else:
                c = col + p * 2
                r = row
            tok = element(c, r)
            if tok >= '1' and tok < '9':
                e = (line.intersect(c * 30 + 40, r * 50 + 40, 55), 'b')
                if e[0] != 9999:
                    point(ctx, c, r, 20)
                t.append(e)
            if tok == '0':
                t.append(line.block(c * 30 - 14, r * 50 - 7, 110, 95))
        t = (9999, 'o') if len(t) == 0 else min(t)
        if t[0] == 9999:
            continue
        if t[1] != 'b':
            line.direction = t[1]
            dist = t[0]
            return None
        t = t[0]
        x, y = line.move(t)
        col, row = grid(x, y)
        t1 = 9999
        t2 = 9999
        ctx.set_source_rgb(0, 0, 0)
        tok = element(col, row)
        if tok < '0' or tok > '9':
            t1 = distance(x, y, col * 30 + 40, row * 50 + 40)
        tok = element(col - 2, row)
        if tok < '0' or tok > '9':
            t2 = distance(x, y, col * 30 - 20, row * 50 + 40)
        ctx.set_source_rgb(1, 0, 0)
        c = col if t1 < t2 else col - 2
        return (c, row)


def cannon_line(ctx, d):
    'Draw a line through the current trajectory'
    ctx.move_to(line.x, line.y)
    ctx.set_source_rgb(.5, .5, 0)
    p = line.move(d)
    ctx.line_to(*p)
    ctx.stroke()


def grid(x, y):
    'Get grid column, row for the point; column-2 could also match'
    c = int((x - 10) / 30)
    r = int((y - 10) / 50)
    if r % 2 != c % 2:
        c += 1
    return (c, r)


def element(c, r):
    'Get the current board value of the given column and row'
    if r < 0 or c < 0:
        return ' '
    try:
        return level[r][c]
    except IndexError:
        return ' '


def point(ctx, c, r, s=2):
    'draw a point, or a small circle'
    ctx.arc(c * 30 + 40, r * 50 + 40, s, 0.0, 2.0 * math.pi)
    ctx.fill()


def line_to(ctx, c, r):
    'draw a line towards a column/row point'
    ctx.line_to(c * 30 + 40, r * 50 + 40)


def show(ctx, img, x, y):
    'Show an image'
    ctx.set_source_surface(img, x, y)
    ctx.paint()


def cb_click(x, y):
    """ Handle a mouseclick
        divide in parts of 60.. find potential hits,
            calculate place of hit on the line
        keep the same color in the different levels, the color is the seed
    """
    global line, dist
    mouse = interface.window.get_pointer()
    line = Line(x=max_x / 2, y=max_y - 56,
                dx=mouse[0] - max_x * sc / 2,
                dy=mouse[1] - (max_y - 56) * sc)
    line.normalize(50)
    dist = line.boxed(35, 35, max_x - 35, max_y - 35)


def cb_timer(area):
    'timer function, move cannon to mouse'
    interface.redraw_all()


def image(name):
    'Read an image file'
    return cairo.ImageSurface.create_from_png(name)


def images(*imgs):
    'Read image files'
    l = []
    for im in imgs:
        l.append(image(im))
    return l

balls = images("ice.png", "red.png", "green.png", "purple.png", "yellow.png",
               "black.png", "grey.png")
star = image("star.png")
cannon = image("cannon.png")

f = file("level.txt", 'r')
level = f.readlines()

interface.register(None, cb_click, cb_render, cb_timer)
interface.drawingArea()
measure()
interface.main()
